import os
import os.path
import sys

from django.views.generic.edit import FormView
from django.views.decorators.csrf import csrf_exempt
from django.shortcuts import render, redirect
from django.contrib.auth import get_user
from django.http import HttpResponse, Http404, JsonResponse
from django.db import transaction

from rest_framework.parsers import JSONParser
from rest_framework.decorators import api_view
from rest_framework.response import Response

from galeria.settings import GALLERY_INFO, LANGUAGE_CODE

from fotos.models import Album, Photo, UploadedImage
from fotos.forms import UploadedImageForm
from fotos.util import get_image_metadata, normalize
from fotos.serializers import AlbumSerializer, PhotoSerializer, \
    UploadedImageSerializer

from datetime import datetime
from babel.dates import get_month_names

def get_month_name(ts, capitalize=False):
    locale = LANGUAGE_CODE.replace('-', '_')
    name = get_month_names(locale=locale)[ts.month]
    if capitalize:
        name = name.capitalize()
    return name

def _get_year_album(year):
    return Album.objects.filter(the_parent=None, slug=year).first()

def _get_month_album(ya, month):
    return Album.objects.filter(the_parent=ya, slug=month).first()

def _get_event_album(ma, event):
    return Album.objects.filter(the_parent=ma, slug=event).first()

def _get_photo(ea, photo):
    return Photo.objects.filter(the_album=ea, basename=photo).first()

def _get_years():
    years = Album.objects.filter(the_parent=None).order_by('first_timestamp')
    if len(years) == 0:
        return None
    return years

def _get_months(ya):
    return Album.objects.filter(the_parent=ya).order_by('first_timestamp')

def _get_events(ma):
    return Album.objects.filter(the_parent=ma).order_by('first_timestamp')

def _get_photos(ea):
    return Photo.objects.filter(the_album=ea).order_by('timestamp')

def _get_prev_year_url(ya):
    prev_year = _get_prev_year(ya)
    if prev_year is not None:
        return prev_year.get_url_path()
    return None

def _get_next_year_url(ya):
    next_year = _get_next_year(ya)
    if next_year is not None:
        return next_year.get_url_path()
    return None

def _get_prev_month_url(ma):
    prev_month = _get_prev_month(ma)
    if prev_month is not None:
        return prev_month.get_url_path()
    return None

def _get_next_month_url(ma):
    next_month = _get_next_month(ma)
    if next_month is not None:
        return next_month.get_url_path()
    return None

def _get_prev_event_url(ea):
    prev_event = _get_prev_event(ea)
    if prev_event is not None:
        return prev_event.get_url_path()
    return None

def _get_next_event_url(ea):
    next_event = _get_next_event(ea)
    if next_event is not None:
        return next_event.get_url_path()
    return None

def _get_prev_photo_url(ph):
    prev_photo = _get_prev_photo(ph)
    if prev_photo is not None:
        return prev_photo.get_url_path()
    return None

def _get_next_photo_url(ph):
    next_photo = _get_next_photo(ph)
    if next_photo is not None:
        return next_photo.get_url_path()
    return None

def _get_prev_year(ya):
    years = _get_years()
    if years is None or years.first() == ya:
        return None
    i = 1
    while years[i] != ya:
        i += 1
    return years[i-1]

def _get_next_year(ya):
    years = _get_years()
    if years is None or years.last() == ya:
        return None
    i = years.count() - 2
    while years[i] != ya:
        i -= 1
    return years[i+1]

def _get_prev_month(ma):
    ya = ma.the_parent
    months = _get_months(ya)
    if ma == months.first():
        prev_year = _get_prev_year(ya)
        if prev_year is None:
            return None
        prev_year_months = _get_months(prev_year)
        return prev_year_months.last()
    i = 1
    while months[i] != ma:
        i += 1
    return months[i-1]

def _get_next_month(ma):
    ya = ma.the_parent
    months = _get_months(ya)
    if ma == months.last():
        next_year = _get_next_year(ya)
        if next_year is None:
            return None
        next_year_months = _get_months(next_year)
        return next_year_months.first()
    i = months.count() - 2
    while months[i] != ma:
        i -= 1
    return months[i+1]

def _get_prev_event(ea):
    ma = ea.the_parent
    events = _get_events(ma)
    if ea == events.first():
        prev_month = _get_prev_month(ma)
        if prev_month is None:
            return None
        prev_month_events = _get_events(prev_month)
        return prev_month_events.last()
    i = 1
    while events[i] != ea:
        i += 1
    return events[i-1]

def _get_next_event(ea):
    ma = ea.the_parent
    events = _get_events(ma)
    if ea == events.last():
        next_month = _get_next_month(ma)
        if next_month is None:
            return None
        next_month_events = _get_events(next_month)
        return next_month_events.first()
    i = events.count() - 2
    while events[i] != ea:
        i -= 1
    return events[i+1]

def _get_prev_photo(ph):
    ea = ph.the_album
    photos = _get_photos(ea)
    if ph == photos.first():
        prev_event = _get_prev_event(ea)
        if prev_event is None:
            return None
        prev_event_photos = _get_photos(prev_event)
        return prev_event_photos.last()
    i = 1
    while photos[i] != ph:
        i += 1
    return photos[i-1]

def _get_next_photo(ph):
    ea = ph.the_album
    photos = _get_photos(ea)
    if ph == photos.last():
        next_event = _get_next_event(ea)
        if next_event is None:
            return None
        next_event_photos = _get_photos(next_event)
        return next_event_photos.first()
    i = photos.count() - 2
    while photos[i] != ph:
        i -= 1
    return photos[i+1]

def main(request):
    page_title = 'Años'
    albums = _get_years()
    if albums is None:
        page_title = 'La galería está vacía'
    attrs = { 'albums': albums,
              'breadcrumbs': [],
              'gallery': GALLERY_INFO,
              'navigation': { 'prev': None, 'next': None, 'up': None },
              'now_year': datetime.now().year,
              'page_title': page_title,
              'prefix': '/galeria',
              'user': get_user(request) }
    return render(request, 'album.html', attrs)

def year(request, year):
    ya = _get_year_album(year)
    if ya is None:
        raise Http404(year + ' does not exist')
    attrs = { 'album': ya,
              'albums': _get_months(ya),
              'breadcrumbs': [ { 'link': ya.get_url_path(),
                                 'name': 'Año ' + year } ],
              'gallery': GALLERY_INFO,
              'navigation': { 'prev': _get_prev_year_url(ya),
                              'next': _get_next_year_url(ya),
                              'up': '/galeria' },
              'now_year': datetime.now().year,
              'page_title': 'Año ' + year,
              'prefix': '/galeria/' + year,
              'user': get_user(request) }
    return render(request, 'album.html', attrs)

def month(request, year, month):
    ya = _get_year_album(year)
    ma = _get_month_album(ya, month)
    if ya is None or ma is None:
        raise Http404(os.path.join(year, month) + ' does not exist')
    attrs = { 'album': ma,
              'albums': _get_events(ma),
              'breadcrumbs': [ { 'link': ya.get_url_path(),
                                 'name': ya.name },
                               { 'link': ma.get_url_path(),
                                 'name': ma.name }  ],
              'gallery': GALLERY_INFO,
              'navigation': { 'prev': _get_prev_month_url(ma),
                              'next': _get_next_month_url(ma),
                              'up':  ya.get_url_path() },
              'now_year': datetime.now().year,
              'page_title': ('Mes de %s, %d' % \
                             ( get_month_name(ma.first_timestamp),
                               ya.first_timestamp.year )),
              'prefix': os.path.join('/galeria/', year, month),
              'user': get_user(request) }
    return render(request, 'album.html', attrs)

def event(request, year, month, event):
    ya = _get_year_album(year)
    ma = _get_month_album(ya, month)
    ea = _get_event_album(ma, event)
    if ya is None or ma is None or ea is None:
        raise Http404(os.path.join(year, month, event) + ' does not exist')
    attrs = { 'album': ea,
              'breadcrumbs': [ { 'link': ya.get_url_path(),
                                 'name': ya.name },
                               { 'link': ma.get_url_path(),
                                 'name': ma.name },
                               { 'link': ea.get_url_path(),
                                 'name': ea.name } ],
              'gallery': GALLERY_INFO,
              'navigation': { 'prev': _get_prev_event_url(ea),
                              'next': _get_next_event_url(ea),
                              'up':  ma.get_url_path() },
              'now_year': datetime.now().year,
              'page_title': ea.name,
              'photos': _get_photos(ea),
              'prefix': os.path.join('/galeria', year, month, event),
              'user': get_user(request) }
    return render(request, 'album.html', attrs)

def photo(request, year, month, event, photo):
    ya = _get_year_album(year)
    ma = _get_month_album(ya, month)
    ea = _get_event_album(ma, event)
    ph = _get_photo(ea, photo)
    if ya is None or ma is None or ea is None or ph is None:
        raise Http404(os.path.join(year, month, event, photo +
                                   ' does not exist'))
    attrs = { 'breadcrumbs': [ { 'link': ya.get_url_path(),
                                 'name': ya.name },
                               { 'link': ma.get_url_path(),
                                 'name': ma.name },
                               { 'link': ea.get_url_path(),
                                 'name': ea.name },
                               { 'link': ph.get_url_path(),
                                 'name': ph.title } ],
              'gallery': GALLERY_INFO,
              'navigation': { 'prev': _get_prev_photo_url(ph),
                              'next': _get_next_photo_url(ph),
                              'up':  ea.get_url_path() },
              'now_year': datetime.now().year,
              'page_title': ph.title,
              'photo': ph,
              'prefix': os.path.join('/galeria', year, month, event),
              'user': get_user(request) }
    return render(request, 'photo.html', attrs)

@transaction.atomic
def _get_photo_album(metadata):
    ts = metadata['timestamp']
    sy = '%04d' % ts.year
    sm = '%02d' % ts.month
    year_album = Album.objects.filter(the_parent=None, slug=sy).first()
    if year_album is None:
        year_album = Album(the_parent = None,
                           name = 'Año ' + sy,
                           slug = sy,
                           first_timestamp = ts,
                           last_timestamp = ts,
                           time_offset = metadata['time_offset'],
                           the_photo = None)
        year_album.save()
    month_album = Album.objects.filter(the_parent=year_album, slug=sm).first()
    if month_album is None:
        month_album = Album(the_parent = year_album,
                            name = get_month_name(ts, capitalize=True),
                            slug = sm,
                            first_timestamp = ts,
                            last_timestamp = ts,
                            time_offset = metadata['time_offset'],
                            the_photo = None)
        month_album.save()
    pa_slug = normalize(metadata['album'])
    event_album = Album.objects.filter(the_parent=month_album,
                                       slug=pa_slug).first()
    if event_album is None:
        event_album = Album(the_parent = month_album,
                            name = metadata['album'],
                            slug = pa_slug,
                            first_timestamp = ts,
                            last_timestamp = ts,
                            time_offset = metadata['time_offset'],
                            the_photo = None)
        event_album.save()
    return year_album, month_album, event_album

@transaction.atomic
def _add_photo(uploaded_image):
    path = '/' + uploaded_image.files.name
    uploaded_image.delete()
    md = get_image_metadata(path)
    if md is None:
        os.remove(path)
        return
    year_album, month_album, event_album = _get_photo_album(md)
    photo = Photo(title = md['title'],
                  description = md['description'],
                  the_album = event_album,
                  location = md['location'],
                  latitude = md['latitude'],
                  longitude = md['longitude'],
                  width = md['width'],
                  height = md['height'],
                  view_width = md['view_width'],
                  view_height = md['view_height'],
                  thumb_width = md['thumb_width'],
                  thumb_height = md['thumb_height'],
                  timestamp = md['timestamp'],
                  time_offset = md['time_offset'])
    photo.fill_aux_data()
    photo.save()
    if event_album.first_timestamp > photo.timestamp:
        event_album.first_timestamp = photo.timestamp
    if event_album.last_timestamp < photo.timestamp:
        event_album.last_timestamp = photo.timestamp
    if event_album.the_photo is None:
        event_album.the_photo = photo
    event_album.save()
    if month_album.first_timestamp > event_album.first_timestamp:
        month_album.first_timestamp = event_album.first_timestamp
    if month_album.last_timestamp < event_album.last_timestamp:
        month_album.last_timestamp = event_album.last_timestamp
    if month_album.the_photo is None:
        month_album.the_photo = photo
    month_album.save()
    if year_album.first_timestamp > month_album.first_timestamp:
        year_album.first_timestamp = month_album.first_timestamp
    if year_album.last_timestamp < month_album.last_timestamp:
        year_album.last_timestamp = month_album.last_timestamp
    if year_album.the_photo is None:
        year_album.the_photo = photo
    year_album.save()
    os.remove(path)

def upload(request):
    if request.method == 'POST':
        form = UploadedImageForm(request.POST, request.FILES)
        files = request.FILES.getlist('files')
        if form.is_valid():
            for f in files:
                uploaded_image = UploadedImage(files=f)
                uploaded_image.save()
                _add_photo(uploaded_image)
            return JsonResponse({'data': 'Data uploaded'})
        else:
            return JsonResponse({'data': 'Something went wrong'})
    else:
        form = UploadedImageForm()
        return render(request, 'upload.html',
                      { 'form': form,
                        'breadcrumbs': [],
                        'gallery': GALLERY_INFO,
                        'navigation': { 'prev': None,
                                        'next': None,
                                        'up': None },
                        'now_year': datetime.now().year,
                        'page_title': 'Subir fotos' })

def highlight_month(request, year, month):
    ya = _get_year_album(year)
    ma = _get_month_album(ya, month)
    if ya is None or ma is None:
        raise Http404(os.path.join(year, month) + ' does not exist')

    if ya.the_photo == ma.the_photo:
        return redirect(os.path.join('/galeria', year, month))

    ya.the_photo = ma.the_photo
    ya.save()
    return redirect(os.path.join('/galeria', year))

def highlight_event(request, year, month, event):
    ya = _get_year_album(year)
    ma = _get_month_album(ya, month)
    ea = _get_event_album(ma, event)
    if ya is None or ma is None or ea is None:
        raise Http404(os.path.join(year, month, event) + ' does not exist')

    if ma.the_photo == ea.the_photo:
        return redirect(os.path.join('/galeria', year, month, event))

    old_ph = ma.the_photo
    if ya.the_photo == old_ph:
        ya.the_photo = ea.the_photo
        ya.save()
    ma.the_photo = ea.the_photo
    ma.save()
    return redirect(os.path.join('/galeria', year, month))

def highlight_photo(request, year, month, event, photo):
    ya = _get_year_album(year)
    ma = _get_month_album(ya, month)
    ea = _get_event_album(ma, event)
    ph = _get_photo(ea, photo)
    if ya is None or ma is None or ea is None or ph is None:
        raise Http404(os.path.join(year, month, event, photo) +
                      ' does not exist')

    if ea.the_photo == ph:
        return redirect(os.path.join('/galeria', year, month, event, photo))

    old_ph = ea.the_photo
    if ya.the_photo == old_ph:
        ya.the_photo = ph
        ya.save()
    if ma.the_photo == old_ph:
        ma.the_photo = ph
        ma.save()
    ea.the_photo = ph
    ea.save()
    return redirect(os.path.join('/galeria', year, month, event))

@api_view(['GET'])
def album_list(request):

    if request.method == 'GET':
        albums = Album.objects.all()
        serializer = AlbumSerializer(albums, many=True)
        return Response(serializer.data)

@api_view(['GET'])
def photo_list(request):

    if request.method == 'GET':
        photos = Photo.objects.all()
        serializer = PhotoSerializer(photos, many=True)
        return Response(serializer.data)

@api_view(['POST'])
def upload_image(request):

    if request.method == 'POST':
        form = UploadedImageSerializer(request.POST, request.FILES)
        files = request.FILES.getlist('files')
        if form.is_valid():
            for f in files:
                uploaded_image = UploadedImage(files=f)
                uploaded_image.save()
                _add_photo(uploaded_image)
            return JsonResponse({'response':'OK'})
        else:
            return JsonResponse({'response':'Error'})
